import { jsx as _jsx } from "react/jsx-runtime";
import { isJsonError, isParentNode, isValueNode } from 'headless-json-editor';
import { renderHook, act, render } from '@testing-library/react';
import { strict as assert } from 'assert';
import { useEditor } from './useEditor';
import { StrictMode } from 'react';
describe('useEditor', () => {
    it('should return state and editor instance', () => {
        const options = {
            schema: { type: 'object', properties: { title: { type: 'string' } } },
            data: { title: 'test-title' }
        };
        const { result } = renderHook((settings) => useEditor(settings), {
            initialProps: options
        });
        assert(result.current != null, 'should have returned instance on final render');
        assert.equal(typeof result.current.draft.getSchema, 'function');
    });
    it('should update state when input data changes', () => {
        const data = { title: 'test-title' };
        const schema = { type: 'object', properties: { title: { type: 'string' } } };
        const { result, rerender } = renderHook((settings) => useEditor(settings), {
            initialProps: { schema, data }
        });
        const editor = result.current;
        assert(editor != null);
        const state = editor.getNode();
        const titleNode = editor.getNode('/title');
        rerender({ schema, data: { title: 'updated title' } });
        const editorAfter = result.current;
        assert(editorAfter != null);
        const stateAfter = editorAfter.getNode();
        const titleNodeAfter = editor.getNode('/title');
        assert(!isJsonError(titleNode) && !isJsonError(titleNodeAfter));
        assert.equal(editor, editorAfter, 'should not recreate editor when data has changed');
        assert(state !== stateAfter);
        assert(isValueNode(titleNode) && isValueNode(titleNodeAfter));
        assert.equal(titleNode.value, 'test-title');
        assert.equal(titleNodeAfter.value, 'updated title');
    });
    it('should update state when input schema changes', () => {
        const data = { title: 'test-title' };
        const schema = { type: 'object', properties: { title: { type: 'string' } } };
        const { result, rerender } = renderHook((settings) => useEditor(settings), {
            initialProps: { schema, data }
        });
        const editor = result.current;
        assert(editor);
        const state = editor.getNode();
        assert(isParentNode(state));
        const titleNode = editor.getNode('/title');
        rerender({
            schema: {
                type: 'object',
                required: ['title', 'subtitle'],
                properties: {
                    title: { type: 'string', minLength: 1 },
                    subtitle: { type: 'string', minLength: 1, default: 'subtitle' }
                }
            },
            data
        });
        const editorAfter = result.current;
        assert(editorAfter);
        const stateAfter = editor.getNode();
        assert(isParentNode(stateAfter));
        const titleNodeAfter = editor.getNode('/title');
        assert(!isJsonError(titleNode) && !isJsonError(titleNodeAfter));
        assert.equal(editor, editorAfter, 'should not recreate editor when schema has changed');
        assert.equal(state.children.length, 1);
        assert.equal(stateAfter.children.length, 2, 'should have updated data after schema change');
        assert.equal(titleNode.schema.minLength, undefined);
        assert.equal(titleNodeAfter.schema.minLength, 1);
    });
    it('should recreated editor when cacheKey changes', async () => {
        const props = {
            data: { title: 'test-title' },
            schema: { type: 'object', properties: { title: { type: 'string' } } }
        };
        const { result, rerender } = renderHook((settings) => useEditor(settings), {
            initialProps: props
        });
        const editor = result.current;
        rerender({ ...props, cacheKey: 123 });
        const editorAfter = result.current;
        assert(editor && editorAfter);
        assert.notEqual(editor, editorAfter);
        assert.notEqual(editor.id, editorAfter.id);
    });
    it('should add internal onChangePlugin', () => {
        const { result } = renderHook((settings) => useEditor(settings), {
            initialProps: {
                data: { title: 'test-title' },
                schema: { type: 'object', properties: { title: { type: 'string' } } }
            }
        });
        const editor = result.current;
        assert(editor);
        assert(editor.plugins.find((p) => p.id === 'InternalOnChange'));
    });
    it('should return latest list of errors', () => {
        const { result } = renderHook((settings) => useEditor(settings), {
            initialProps: {
                data: { title: 'test-title' },
                schema: { type: 'object', properties: { title: { type: 'string', minLength: 2 } } }
            }
        });
        const editor = result.current;
        assert(editor);
        assert.equal(editor.getErrors().length, 0);
        act(() => {
            editor.setValue('#/title', 'X');
        });
        const editorAfter = result.current;
        assert(editorAfter);
        const titleNode = editorAfter.getNode('#/title');
        assert.equal(titleNode.type, 'string');
        assert.equal(titleNode.value, 'X');
        assert.equal(editor.getErrors().length, 1);
    });
    it('should not recreate editor using setValue', () => {
        const { result } = renderHook((settings) => useEditor(settings), {
            initialProps: {
                data: { title: 'test-title' },
                schema: { type: 'object', properties: { title: { type: 'string', minLength: 2 } } }
            }
        });
        const editorBefore = result.current;
        assert(editorBefore);
        act(() => {
            editorBefore.setValue('#/title', 'X');
        });
        const editorAfter = result.current;
        assert(editorAfter);
        assert.equal(editorAfter.getData().title, 'X');
        assert.equal(editorBefore.getErrors().length, 1);
        assert(editorBefore === editorAfter);
    });
    it('should rerender after validation', () => {
        const renderCalls = [];
        const options = {
            data: { title: '' },
            schema: { type: 'object', properties: { title: { type: 'string', minLength: 2 } } },
            validate: false
        };
        function Form() {
            const editor = useEditor(options);
            renderCalls.push(editor);
            return _jsx("div", { children: editor === null || editor === void 0 ? void 0 : editor.id });
        }
        render(_jsx(Form, {}));
        const editor = renderCalls[renderCalls.length - 1];
        assert(editor);
        assert.equal(editor.getErrors().length, 0);
        renderCalls.length = 0;
        act(() => {
            editor.validate();
        });
        assert.equal(renderCalls.length, 1);
        const editorAfter = renderCalls[renderCalls.length - 1];
        assert.equal(editorAfter === null || editorAfter === void 0 ? void 0 : editorAfter.getErrors().length, 1);
    });
    describe('destroy', () => {
        it('should destroy editor on unmount', () => {
            let currentEditor = null;
            function Form() {
                const editor = useEditor({
                    schema: { type: 'object' },
                    widgets: [{ id: 'dummy', use: () => true, Widget: () => _jsx("div", {}) }]
                });
                currentEditor = editor;
                if (editor == null) {
                    return null;
                }
                const root = editor.getNode();
                const Widget = editor.getWidget(root);
                return _jsx(Widget, { node: root, editor: editor });
            }
            const { unmount } = render(_jsx(Form, {}));
            assert(currentEditor != null);
            currentEditor = currentEditor;
            unmount();
            assert(currentEditor != null);
            assert(currentEditor.plugins == null);
            assert(currentEditor.root == null);
        });
        it('should call destroy for each created instance in StrictMode', () => {
            const createCalls = [];
            const destroyCalls = [];
            function Form() {
                const editor = useEditor({
                    schema: { type: 'object' },
                    plugins: [
                        () => {
                            createCalls.push(new Date());
                            return {
                                id: 'spy',
                                onEvent() { },
                                onDestroy() {
                                    destroyCalls.push(new Date());
                                }
                            };
                        }
                    ],
                    widgets: [{ id: 'dummy', use: () => true, Widget: () => _jsx("div", {}) }]
                });
                if (editor == null) {
                    return null;
                }
                const root = editor.getNode();
                const Widget = editor.getWidget(root);
                return _jsx(Widget, { node: root, editor: editor });
            }
            const { rerender } = render(_jsx(StrictMode, { children: _jsx(Form, {}) }));
            rerender(_jsx(StrictMode, { children: _jsx("div", {}) }));
            assert.equal(createCalls.length, destroyCalls.length);
        });
    });
    describe('options: extendDefaults', () => {
        it('should set `extendDefault` in draft to false', () => {
            var _a;
            const { result } = renderHook((settings) => useEditor(settings), {
                initialProps: {
                    extendDefaults: false,
                    schema: { type: 'array' }
                }
            });
            const editor = result.current;
            assert.equal((_a = editor === null || editor === void 0 ? void 0 : editor.draft.config.templateDefaultOptions) === null || _a === void 0 ? void 0 : _a.extendDefaults, false);
        });
        it('should set `extendDefault` in draft to true', () => {
            var _a;
            const { result } = renderHook((settings) => useEditor(settings), {
                initialProps: {
                    extendDefaults: true,
                    schema: { type: 'array' }
                }
            });
            const editor = result.current;
            assert.equal((_a = editor === null || editor === void 0 ? void 0 : editor.draft.config.templateDefaultOptions) === null || _a === void 0 ? void 0 : _a.extendDefaults, true);
        });
        it('should not add minimum required array items if default-value is []', () => {
            var _a;
            const { result } = renderHook((settings) => useEditor(settings), {
                initialProps: {
                    schema: { type: 'array', minItems: 1, default: [], items: { type: 'string' } }
                }
            });
            const state = (_a = result.current) === null || _a === void 0 ? void 0 : _a.getNode();
            assert.equal(state === null || state === void 0 ? void 0 : state.type, 'array');
            assert.equal(state.children.length, 0);
        });
        it('should not add minimum required array items if default-value is [] (2)', () => {
            var _a;
            const { result } = renderHook((settings) => useEditor(settings), {
                initialProps: {
                    schema: {
                        required: ['list'],
                        type: 'object',
                        properties: {
                            list: {
                                type: 'array',
                                minItems: 1,
                                default: [],
                                items: { type: 'string' }
                            }
                        }
                    }
                }
            });
            const list = (_a = result.current) === null || _a === void 0 ? void 0 : _a.getNode('/list');
            assert.equal(list === null || list === void 0 ? void 0 : list.type, 'array');
            assert.equal(list.children.length, 0);
        });
    });
});
//# sourceMappingURL=useEditor.test.js.map